
/*
   Sketch to use NodeMCU ESP8266 to test record and display results of pings to Internet and local network devices
   This sketch uses the ESP8266WiFi library and NTPClient which provides time functions
   Results can be recovered by FTP from the SPIFFS record filed by YYYY_MM.txt
   
*/

/*
 * TESTING: 
 *  in netmonitorV2.h change URL, interval and nRounds 
 *  int URL = 4; // *** if = 4 uses LP to check operation of test;
 *  //int URL = 6; //  if = 6 uses talktalk service to check web availability
 * const long interval = 60000;   //60 seconds betwen successive rounds
 * const int nRounds = 60; //number of full rounds to complete before calling ntp & save to file: 60 rounds * interval (60 sec) = 1 hour
 *
 * //const long interval = 6000;//for testing
 * //const int nRounds = 10; //for testing: number of full rounds to complete before calling ntp & save to file: 10 rounds * interval (6 sec) = 1 min
 */

 bool diagnostic = true; //false; //if "true" prints diagnostic messages from spiffs tab - lines to send to spiffs
 bool diagnostic_serial = false; //false; //if "true" prints diagnostic messages in serial tab -headline, statsline
 bool diagnostic_pings = false; //false; //if "true" prints diagnostic messages in pings tab - warnlevel
 
#include <ESP8266WiFi.h>            //WiFi functions
#include <time.h>  // time() ctime() 
//#include <WiFiUdp.h>                //NTP uses the User Datagram Protocol (UDP) 
//#include <NTPClient.h>              //time functions sync to NTP: no longer needed as now part of core
#include <Pinger.h>                 //handles the ping fuctions
#include <core_esp8266_waveform.h>  //pulse waveforms for LEDs
#include <FS.h>                     //provides the SPIFFS library
#include <ESP8266FtpServer.h>       //ftp server library
#include "netmonitorV3.h"           //declarations
#include "creds.h"                  // Replace with your network credentials

/* Configuration of NTP */
#define MY_NTP_SERVER "pool.ntp.org"         
#define MY_TZ "GMT0"

/* Globals */
time_t now;                         // this is the epoch
tm tm;                              // the structure tm holds time information in a more convient way

// Set global to avoid object removing after setup() routine
Pinger pinger;

//function prototypes in seperate tabs
//leds
void flashLeds(int reps, int rate);  //at start, test all LEDs working
void showLEDs();  // display ping results to LEDs
//handling ping tests
bool PerformPing();  // Pings a single address host1[target] with a set of pingNumber pings and timeout pingDelay
void checkFailedPings(); //tests & activates warning if internet failure
//serial
void printResults(); // for each completed round of pings print results to serial: note stats line is built differently for end of round averages
void listStats(); //prepare header line for printing also prepare stats line for printing.
//spiffs
void sumTotals();    //aggregate results across the whole set of pings for each target
void zeroStats(); //ready for first or next round
void prepStats(); //prepare header line for filing also prepare stats line for filing.
void writeFile(); //write stats; and header line only if its a new day or first readings; if new day reset warnLevel & cancel internet fail led on D8 (D8=0)
void logRestart(); // log and serial print a restart has occurred - possible mains failure
void logWebState(); //write web failure or web restored message to file
//wifi
void connectWiFi();  // 
//time
void buildStrings(); //creates dayStamp and timeStamp and updates integers YYYY, MM, DD, hh, mm, ss




void setup()
{
  // Begin serial connection
  Serial.begin(115200);
  delay(200); // serial comms is disturbed during a restart
  Serial.print("Connecting to ");
  Serial.println(ssid);
   
  configTime(MY_TZ, MY_NTP_SERVER); // --> Here is the IMPORTANT ONE LINER needed in your sketch!

  //set up LED pins as outputs & test them
  for (int i = 0; i <= 9; i++) {  pinMode(dPin[i], OUTPUT); }
  flashLeds(4, 250);

  //monitor Wifi connection and handle D0   WiFi indicator
  connectWiFi();
  
  gotIpEventHandler = WiFi.onStationModeGotIP([](const WiFiEventStationModeGotIP & event)
  {
    //Serial.print("Station connected, IP: ");
    //Serial.println(WiFi.localIP());
    WiFiState = 1;
    digitalWrite(dPin[0], 0); //blue LED is active LOW
  });

  disconnectedEventHandler = WiFi.onStationModeDisconnected([](const WiFiEventStationModeDisconnected & event)
  {
    Serial.println("Station disconnected");
    WiFiState = 0;
    digitalWrite(dPin[0], 1); //blue LED OFF
  });


  buildStrings();  //get myFormattedDate for printing
  
  today = DD;  //dont print new header line to file if its just started. today gets reset in spiffs:writeFile
  thisMonth = MM;

  //FTP Setup, ensure SPIFFS is started before ftp;
  if (SPIFFS.begin()) {
    Serial.println("SPIFFS opened!");
    ftpSrv.begin("esp8266", "esp8266");   //username, password for ftp.  set ports in ESP8266FtpServer.h  (default 21, 50009 for PASV)
  }

  zeroStats(); //initialise the statistics arrays
  roundComplete = true;  //ready to start a new set of pings
  startTime = millis(); //handling frequency that ping set is repeated

  logRestart(); //having given time to start spiffs
  warnLevel = 120; //at restart put red LED on to show a fault has occurred: it should stay on for 20 min
  
  //pinger.SetEchoPayloadLength(256); // Sets echo payload length, in bytes: default is 32 which is standard
  pinger.OnReceive([](const PingerResponse & response)
  {
    return true;  // Return true to continue the ping sequence. If current event returns false, the ping sequence is interrupted.
  });

  pinger.OnEnd([](const PingerResponse & response)
  {
    pingAvg = int(response.AvgResponseTime + 0.5); //ensure pingAvg can not be zero as zero will be used to flag no response.
    pingsLost = response.TotalSentRequests - response.TotalReceivedResponses;
    if (pingsLost < 0) pingsLost = 0; //handle occasional incorrect negative value
    isFinished = true;  //marks all pings to that target completed
    return true;
  });
  target = 0;
}


void loop() {
  ftpSrv.handleFTP();        //make sure in loop you call handleFTP()!!

  switch (target) {
    case 0 ... 6 : {
        if (PerformPing()) target++; //pings tab;   if 1 round is complete, ping next target until all seven have been pinged
      }
      break;

    case 7: {  //one round of pings complete
        //record WiFi Up time
        if (WiFiState == 0 && WiFiUp > 0)WiFiUp--; //dont decrement below zero!     
        sumTotals(); //SPIFFS tab; aggregate totals ready for filing
        printResults(); //Serial tab;
        showLEDs();  //LEDs tab; show target ping status
        buildStrings(); //time tab;
        target = 8;
      }
      break;

    case 8: { //write averages to serial & file
        if ((count >= nRounds) || (DD != today)) {
          //we have done nRounds of sets of pings to all targets so get date and time
          //or its a new day, start next round immediately so times are near hours.
          listStats();  //Serial tab; print header line and 1 set of stats to serial monitor
          prepStats();  //SPIFFS tab; prepare stats for filing
          zeroStats();  //SPIFFS tab; reset stats to zero
          writeFile();  //write averages to file: & if its a new day updates "today".
          count = 1;
        } else {
          count++;
        }
        target = 9;
      }
      break;


    case 9: { //if its a new month do we need to delete last years file for the same month?
        if (thisMonth != MM) {
          //new month so delete same month last year
          lastYear = String(YYYY - 1);
          String delFileName = "/" + lastYear + "-" + sMonth + ".txt"; //filename MUST be fully qualified ie have a leading slash.
          if (SPIFFS.exists(delFileName)) {
            SPIFFS.remove(delFileName);
          }
        }
        target = 10;
      }
      break;


    case 10: { //is it time for another round?
      }
      if (millis() - startTime > interval) {
        startTime = millis();
        target = 0;
      }
  }
}//loop
